"use strict";

const commonTasks = require('@telerik/kendo-common-tasks');
const path = require('path');
const exec = require('child_process').exec;

const webpackConfig = require('./webpack.config.js');
const apiConfig = require('./api.config.js');
const karmaConfigPath = path.join(__dirname, 'karma.conf.js');
const e2eKarmaConfigPath = path.join(__dirname, 'karma.conf.e2e.js');

const tslint = require('gulp-tslint');
const typedoc = require('gulp-typedoc');
const api = require('./src/api/generator.js');

module.exports = function(gulp, libraryName) {
    commonTasks.addTasks(gulp, libraryName, 'src/**/*.ts', webpackConfig);

    gulp.task('test', (done) =>
        commonTasks.startKarma(done, karmaConfigPath, true));

    gulp.task('e2e', (done) =>
        commonTasks.startKarma(done, e2eKarmaConfigPath, true));

    gulp.task('watch-test', (done) =>
        commonTasks.startKarma(done, karmaConfigPath, false));

    gulp.task('lint', () => {
        return gulp.src([ 'src/**/*.ts', 'test/**/*.ts' ])
            .pipe(tslint())
            .pipe(tslint.report("verbose"));
    });

    gulp.task('api-json', () => {
        return gulp.src(["src/**/*.ts", "typings/index.d.ts"])
            .pipe(typedoc(apiConfig.typedoc));
    });

    gulp.task('api', ['api-json'], (done) => {
        api.generate(done, apiConfig);
    });

    gulp.task('api-check', ['api'], (done) => {
        exec(`git diff --color -- ${apiConfig.outPath}`, (stderr, stdout) => {
            let error;

            if (stdout) {
                error = new Error(`API Reference not up to date\n\n${stdout}`);
            }

            done(error);
        });
    });
};
