"use strict";

const template = require('./template-utils.js');
const slug = require('./slug.js');
const utils = require('./utils.js');

const commentTemplate = template.compileFrom('comment-tags.hbs');
template.partialFrom('comment_example', 'comment-example.hbs');

const commentTags = (comment, member) =>
    commentTemplate({
        tags: comment.tags.map(tag => {
            const isExample = tag.tag === 'example';
            let text = tag.text.trim();
            let caption = '';

            if (isExample) {
                text = text.replace(/@@/g, '@');

                const match = text.match(/<caption>(.+)<\/caption>/);
                if (match && match.length === 2) {
                    text = text.replace(/.*\n/, '');
                    caption = match[1];
                }
            }

            if (tag.tag === 'see') {
                const linkSlug = slug(member.packageName, text);
                text = `\n\nSee [${text}]({% slug ${linkSlug} %})`;
            }

            return {
                isExample: isExample,
                caption: caption,
                text: text
            };
        })
    });

const formatComment = (member, parentName) => {
    const comment = member.comment;
    if (!comment) {
        let source = member.name;
        if (parentName) {
            source = `${parentName}.${member.name}`;
        }

        utils.warn('Undocumented member', source);
        return '';
    }

    let output = comment.shortText || "";
    if (comment.text) {
        output += '\n\n' + comment.text;
    }

    if (comment.tags) {
        output += commentTags(comment, member);
    }

    return output;
};

module.exports = formatComment;

