"use strict";

const template = require('./template-utils.js');
const comment = require('./comment.js');
const mapProps = require('./map-props.js');

const componentTemplate = template.compileFrom('component-page.hbs');
template.partialFrom('member_meta', 'member-meta.hbs');

const selector = (member) => {
    const decorators = member.decorators || [];
    const compDecorator = decorators.find(d =>
        d.name === 'Component' || d.name === 'Directive'
    );

    if (compDecorator) {
        return compDecorator.arguments.obj.match(/selector:\s?['"](.+?)['"]/)[1];
    }
};

const isInput = (prop) =>
    prop.decorators && prop.decorators[0].name === 'Input';

const isEvent = (prop) =>
    prop.type && prop.type.name === 'EventEmitter';

const componentPage = (component) => {
    let inputs;
    let events;
    if (component.children) {
        inputs = mapProps(component.children.filter(isInput), component.name);
        events = mapProps(component.children.filter(isEvent), component.name);
    }

    // TODO: Methods

    return componentTemplate({
        name: component.name,
        selector: selector(component),
        comment: comment(component),
        inputs: inputs,
        events: events,
        slug: component.slug
    });
};

module.exports = componentPage;

