"use strict";

const comment = require('./comment.js');
const types = require('./type-utils.js');

const mapProps = (children, parentName) =>
    children.map(prop => {
        let type;
        if (prop.type) {
            type = types.typeString(prop);
        } else if (prop.kindString === 'Accessor') {
            // Fingers crossed
            type = types.typeString(prop.getSignature ?
                prop.getSignature[0] :
                prop.setSignature[0].parameters[0]
            );
        }

        let typeArgs;
        if (prop.type && prop.type.typeArguments) {
            typeArgs = prop.type.typeArguments.map(arg => types.singleTypeString(arg)).join(', ');
        }

        return {
            name: prop.name,
            type: type,
            typeSlug: prop.typeSlug,
            typeArgs: typeArgs,
            comment: comment(prop, parentName)
        };
    });

module.exports = mapProps;

