"use strict";

const unionTypes = (member) =>
    member.type.types.map(type => {
        switch (type.type) {
        case 'stringLiteral':
            return `"${type.value}"`;
        default:
            return singleTypeString(type);
        }
    });

const signature = (definition) => {
    let params = [];
    if (definition.parameters) {
        params = definition.parameters.map((param) =>
            `${param.name}: ${typeString(param)}`
        );
    }

    const returnType = typeString(definition);

    return {
        params: params,
        returnType: returnType
    };
};

const indexType = (definition) => {
    const sig = signature(definition);
    return `{[${sig.params.join(', ')}]}: ${sig.returnType}}`;
};

const callType = (definition) => {
    const sig = signature(definition);
    return `(${sig.params.join(', ')}) => ${sig.returnType}`;
};

const singleTypeString = (type) => {
    let suffix = '';
    if (type.type === 'reflection') {
        const decl = type.declaration;
        if (decl.signatures) {
            const sig = decl.signatures[0];
            if (decl.name === '__type' && sig.name === '__call') {
                return callType(sig);
            }
        } else if (decl.indexSignature) {
            return indexType(decl.indexSignature[0]);
        }
    }

    if (type.typeArguments) {
        suffix = `<${singleTypeString(type.typeArguments[0])}>`;
    }

    if (type.isArray) {
        suffix = '[]';
    }

    return type.name + suffix;
};

const typeString = (prop) => {
    const type = prop.type;

    if (type.type === 'union') {
        return unionTypes(prop).join(' | ');
    }

    return singleTypeString(prop.type);
};

module.exports = {
    unionTypes: unionTypes,
    typeString: typeString,
    singleTypeString: singleTypeString
};

