const commonTasks = require('@telerik/kendo-common-tasks');
const path = require('path');

const sourceExtensions = [ '.ts' ];
const nodeModulesPath = path.join(__dirname, 'node_modules');

const resolve = commonTasks.resolveConfig(sourceExtensions, nodeModulesPath);

const packageInfo = require(path.join(process.cwd(), 'package.json'));
const packageDependencies = Object.keys(packageInfo["dependencies"] || {})
    .map(dep => new RegExp(`^${dep}$`));

const tsLoader = {
    test: /\.ts?$/,
    exclude: /(node_modules)/,
    loader: require.resolve('ts-loader'),
    query: {
        compilerOptions: {
            sourceMap: true
        }
    }
};

module.exports = {
    npmPackage: commonTasks.webpackThemeConfig({ extract: true }, {
        resolve,

        output: { libraryTarget: 'commonjs' },

        externals: [
            /^\.\//,
            /^\.\.\//
        ].concat(packageDependencies),

        module: {
            loaders: [ tsLoader ]
        },

        devtool: 'source-map'
    }),

    dev: commonTasks.webpackDevConfig({
        resolve,
        loaders: [ tsLoader ],
        entries: 'examples/*.ts'
    })
};
