// @AngularClass

/*
 * Helper: root(), and rootDir() are defined at the bottom
 */
const path = require('path');
// Webpack Plugins
const DefinePlugin = require('webpack/lib/DefinePlugin');
const ENV = process.env.ENV = process.env.NODE_ENV = 'test';
const webpack = require('webpack');

const commonTasks = require('@telerik/kendo-common-tasks');

const sourceExtensions = [ '.ts' ];
const nodeModulesPath = path.join(__dirname, 'node_modules');

const myResolve = commonTasks.resolveConfig(sourceExtensions, nodeModulesPath);

/*
 * Config
 */
module.exports = commonTasks.webpackThemeConfig({ stubResources: true }, {
    resolveLoader: {
        fallback: myResolve.fallback
    },
    resolve: {
        cache: false,
        fallback: myResolve.fallback,
        alias: {
            "./test": process.cwd() + "/test",
            "./e2e": process.cwd() + "/e2e"
        },
        extensions: [ '.ts','.js','.scss', '' ]
    },
    devtool: 'inline-source-map',
    module: {
        preLoaders: [
            {
                test: /\.js$/,
                loader: require.resolve("source-map-loader"),
                exclude: [
                    /node_modules\/rxjs/
                ]
            }
        ],
        loaders: [
            {
                test: /\.ts$/,
                loader: 'ts-loader',
                query: {
                    "compilerOptions": {
                        "removeComments": true
                    }
                },
                exclude: [ /\.e2e\.ts$/, /node_modules/ ]
            }
        ],
        postLoaders: [
            // instrument only testing sources with Istanbul
            {
                test: /\.(js|ts)$/,
                include: root('src'),
                loader: 'istanbul-instrumenter-loader',
                exclude: [
                    /\.e2e\.ts$/,
                    /node_modules/
                ]
            }
        ]
    },
    stats: { colors: true, reasons: true },
    debug: false,
    plugins: [
        new webpack.ContextReplacementPlugin(/\.\/test/, process.cwd() + '/test'),
        new webpack.ContextReplacementPlugin(/\.\/e2e/, process.cwd() + '/e2e'),
        new DefinePlugin({
            // Environment helpers
            'process.env': {
                'ENV': JSON.stringify(ENV),
                'NODE_ENV': JSON.stringify(ENV)
            }
        })
    ],
    // we need this due to problems with es6-shim
    node: {
        global: 'window',
        progress: false,
        crypto: 'empty',
        module: false,
        clearImmediate: false,
        setImmediate: false
    }
});

// Helper functions

function root() {
    const theArgs = Array.prototype.slice.call(arguments, 0);
    return path.join.apply(path, [ __dirname ].concat(theArgs));
}
